﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.IO.Pipes;
using System.IO;
using System.Threading;
using System.Diagnostics;
using NamedPipes20080621.Server.WinUI.Properties;

namespace Benday.NamedPipesSample.WinUI
{
    public partial class NamedPipeUI : Form
    {
        private const string NAME_OF_PIPE = "NamedPipeTest";
        public NamedPipeUI()
        {
            InitializeComponent();
        }

        ~NamedPipeUI()
        {
            if (Server != null) Server.Stop();
            if (Client != null) Client.Stop();
        }

        public NamedPipeServer Server { get; set; }

        private void StartServer()
        {
            Server = new NamedPipeServer(NAME_OF_PIPE);
            Server.OnReceivedMessage += new EventHandler<ReceivedMessageEventArgs>(Server_OnReceivedMessage);
            Server.Start();
        }

        delegate void ReceivedMessageDelegate(object sender, ReceivedMessageEventArgs e);

        private Queue<string> m_fromClientQueue = new Queue<string>();

        void Server_OnReceivedMessage(object sender, ReceivedMessageEventArgs e)
        {
            m_fromClientQueue.Enqueue(e.Message);
        }
        
        private void StopServer()
        {
            if (this.Server != null)
            {
                Server.Stop();
            }
        }

        public NamedPipeClient Client { get; set; }

        private void StartClient()
        {
            Client = new NamedPipeClient(NAME_OF_PIPE);
            Client.OnReceivedMessage += new EventHandler<ReceivedMessageEventArgs>(Client_OnReceivedMessage);
            Client.Start();
        }

        private Queue<string> m_fromServerQueue = new Queue<string>();

        void Client_OnReceivedMessage(object sender, ReceivedMessageEventArgs e)
        {
            m_fromServerQueue.Enqueue(e.Message);
        }

        private void StopClient()
        {
            if (this.Client != null)
            {
                Client.Stop();               
            }
        }

        private void m_btnStartServer_Click(object sender, EventArgs e)
        {
            try
            {
                StartServer();
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.ToString());
            }
        }

        private void m_btnStopServer_Click(object sender, EventArgs e)
        {
            try
            {
                StopServer();
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.ToString());
            } 
        }

        private void m_timerServerReadFromPipe_Tick(object sender, EventArgs e)
        {
            if (m_fromClientQueue.Count > 0)
            {
                string value = m_fromClientQueue.Dequeue();

                m_rtbFromClient.AppendText(value);
                m_rtbFromClient.AppendText(Environment.NewLine);
            }
        }

        private void m_btnStartClient_Click(object sender, EventArgs e)
        {
            try
            {
                StartClient();
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.ToString());
            }
        }

        private void m_btnStopClient_Click(object sender, EventArgs e)
        {
            try
            {
                StopClient();
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.ToString());
            }
        }

        private void m_timerClientReadFromPipe_Tick(object sender, EventArgs e)
        {
            try
            {
                if (m_fromServerQueue.Count > 0)
                {
                    m_rtbFromServer.AppendText(m_fromServerQueue.Dequeue());
                    m_rtbFromServer.AppendText(Environment.NewLine);
                }
            }
            catch (Exception ex)
            {
                m_timerClientReadFromPipe.Enabled = false;
                MessageBox.Show(ex.ToString());
            }
        }

        private void WriteMessageToClient()
        {
            this.Server.Write(DateTime.Now.ToString());
        }

        private void m_btnWriteMessageToClient_Click(object sender, EventArgs e)
        {
            try
            {
                WriteMessageToClient();               
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.ToString());
            } 
        }

        private void WriteMessageToServer()
        {
            this.Client.Write(DateTime.Now.ToString());            
        }

        private void m_btnWriteMessageToServer_Click(object sender, EventArgs e)
        {
            try
            {
                WriteMessageToServer();
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.ToString());
            }
        }

        private void ServerUI_Load(object sender, EventArgs e)
        {
            try
            {
                if (m_chkAutoStartClientAndServer.Checked == true)
                {
                    StartServer();
                    StartClient();
                }
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.ToString());
            } 
        }

        private void NamedPipeUI_FormClosing(object sender, FormClosingEventArgs e)
        {
            Settings.Default.Save();
        }
    }
}
